@@ -16,7 +16,9 @@
 #endif
 Q_QMLTYPE_EXPORT void qml_register_types_org_deepin_launchpad_windowed()
 {
+    QT_WARNING_PUSH QT_WARNING_DISABLE_DEPRECATED
+    QT_WARNING_POP
     qmlRegisterModule("org.deepin.launchpad.windowed", 1, 0);
 }
 
-static const QQmlModuleRegistration registration("org.deepin.launchpad.windowed", qml_register_types_org_deepin_launchpad_windowed);
+static const QQmlModuleRegistration orgdeepinlaunchpadwindowedRegistration("org.deepin.launchpad.windowed", qml_register_types_org_deepin_launchpad_windowed);
