@@ -9,6 +9,10 @@
 
 #include <DConfig>
 #include <DExpected>
+#include <QLoggingCategory>
+
+Q_DECLARE_LOGGING_CATEGORY(logDdeIntegration)
+
 DCORE_USE_NAMESPACE
 
 using AppManager1Application = __AppManager1Application;
@@ -18,7 +22,8 @@ Q_CONSTRUCTOR_FUNCTION(registerComplexDbusType);
 static QString parseDisplayName(const QStringMap &source)
 {
     static QString key = QLocale::system().name();
-    return source.value(key, source.value(u8"default"));
+    const QString & defaultValue = source.value(u8"default");
+    return source.value(key, key.contains('_') ? source.value(key.split('_')[0], defaultValue) : defaultValue);
 }
 
 static QString parseName(const QStringMap &source)
@@ -69,6 +74,7 @@ static AppMgr::AppItem *parseDBus2AppItem(const ObjectInterfaceMap &source)
     }
 
     if (!item) {
+        qCWarning(logDdeIntegration) << "Failed to create AppItem";
         return nullptr;
     }
 
@@ -76,9 +82,15 @@ static AppMgr::AppItem *parseDBus2AppItem(const ObjectInterfaceMap &source)
         item->categories = value.value();
     }
 
-    // fallback to Name if GenericName is empty for X_Deepin_Vendor.
+    // fallback to Name if GenericName is empty, only for X_Deepin_Vendor equals to "deepin".
     const auto deepinVendor = parseDBusField<QString>(appInfo, u8"X_Deepin_Vendor");
-    item->displayName = getDisplayName(deepinVendor && !deepinVendor.value().isEmpty(),
+    item->vendor = deepinVendor ? deepinVendor.value() : QString();
+    
+    // Get GenericName field
+    const auto genericNameMap = parseDBusField<QStringMap>(appInfo, u8"GenericName");
+    item->genericName = genericNameMap ? parseName(genericNameMap.value()) : QString();
+    
+    item->displayName = getDisplayName(deepinVendor && deepinVendor.value() == QStringLiteral("deepin"),
                                        parseDBusField<QStringMap>(appInfo, u8"Name").value(),
                                        parseDBusField<QStringMap>(appInfo, u8"GenericName").value());
 
@@ -98,6 +110,10 @@ static AppMgr::AppItem *parseDBus2AppItem(const ObjectInterfaceMap &source)
         item->lastLaunchedTime = value.value();
     }
 
+    if (auto value = parseDBusField<bool>(appInfo, u8"AutoStart")) {
+        item->isAutoStart = value.value();
+    }
+
     return item;
 }
 
@@ -114,6 +130,7 @@ AppMgr::~AppMgr()
 {
     for (auto item : std::as_const(m_appItems)) {
         if (auto handler = item->handler) {
+            qCDebug(logDdeIntegration) << "Deleting handler for app:" << item->id;
             handler->deleteLater();
         }
     }
@@ -126,7 +143,7 @@ AppManager1Application * createAM1AppIfaceByPath(const QString &dbusPath)
                                                                     dbusPath,
                                                                     QDBusConnection::sessionBus());
     if (!amAppIface->isValid()) {
-        qDebug() << "D-Bus interface not exist or failed to connect to" << dbusPath;
+        qCWarning(logDdeIntegration) << "D-Bus interface not exist or failed to connect to" << dbusPath;
         return nullptr;
     }
 
@@ -137,10 +154,10 @@ AppManager1Application * createAM1AppIface(const QString &desktopId)
 {
     auto appItem = AppMgr::instance()->appItem(desktopId);
     if (!appItem) {
-        qWarning() << "Can't find appItem for the desktopId" << desktopId;
+        qCWarning(logDdeIntegration) << "Can't find appItem for the desktopId" << desktopId;
         return nullptr;
     }
-    qDebug() << "Get app interface for the desktopId" << desktopId;
+    qCDebug(logDdeIntegration) << "Get app interface for the desktopId" << desktopId;
     return appItem->handler;
 }
 
@@ -148,28 +165,35 @@ AppManager1Application * createAM1AppIface(const QString &desktopId)
 // if return true, it means we attempted to launch it via AM, but not sure if it's succeed.
 bool AppMgr::launchApp(const QString &desktopId)
 {
+    qCInfo(logDdeIntegration) << "Launching app:" << desktopId;
     AppManager1Application * amAppIface = createAM1AppIface(desktopId);
-    if (!amAppIface) return false;
+    if (!amAppIface) {
+        qCWarning(logDdeIntegration) << "Failed to get interface for desktop ID:" << desktopId;
+        return false;
+    }
 
     const auto path = amAppIface->path();
     QProcess process;
     process.setProcessChannelMode(QProcess::MergedChannels);
     process.start("dde-am", {"--by-user", path});
     if (!process.waitForFinished()) {
-        qWarning() << "Failed to launch the desktopId:" << desktopId << process.errorString();
+        qCWarning(logDdeIntegration) << "Failed to launch the desktopId:" << desktopId << process.errorString();
         return false;
     } else if (process.exitCode() != 0) {
-        qWarning() << "Failed to launch the desktopId:" << desktopId << process.readAll();
+        qCWarning(logDdeIntegration) << "Failed to launch the desktopId:" << desktopId << process.readAll();
         return false;
     }
-    qDebug() << "Launch the desktopId" << desktopId;
+    qCInfo(logDdeIntegration) << "Successfully launched desktop ID:" << desktopId;
     return true;
 }
 
 bool AppMgr::autoStart(const QString &desktopId)
 {
     AppManager1Application * amAppIface = createAM1AppIface(desktopId);
-    if (!amAppIface) return false;
+    if (!amAppIface) {
+        qCWarning(logDdeIntegration) << "Failed to get interface for desktop ID:" << desktopId;
+        return false;
+    }
 
     return amAppIface->autoStart();
 }
@@ -177,32 +201,82 @@ bool AppMgr::autoStart(const QString &desktopId)
 void AppMgr::setAutoStart(const QString &desktopId, bool autoStart)
 {
     AppManager1Application * amAppIface = createAM1AppIface(desktopId);
-    if (!amAppIface) return;
+    if (!amAppIface) {
+        qCWarning(logDdeIntegration) << "Failed to get interface for desktop ID:" << desktopId;
+        return;
+    }
 
     amAppIface->setAutoStart(autoStart);
+    qCDebug(logDdeIntegration) << "Successfully set autoStart for" << desktopId << "to:" << autoStart;
 }
 
-// 0: global scaleFactor
-double AppMgr::scaleFactor(const QString &desktopId)
+static const QStringList DisabledScaleEnvironments {
+    "DEEPIN_WINE_SCALE=1",
+    "QT_SCALE_FACTOR=1",
+    "GDK_SCALE=1",
+    "GDK_DPI_SCALE=1",
+    "D_DXCB_DISABLE_OVERRIDE_HIDPI=1"
+};
+
+bool AppMgr::disableScale(const QString &desktopId)
 {
     AppManager1Application * amAppIface = createAM1AppIface(desktopId);
-    if (!amAppIface) return 0;
+    if (!amAppIface) {
+        qCWarning(logDdeIntegration) << "Failed to get interface for desktop ID:" << desktopId;
+        return false;
+    }
 
-    return amAppIface->scaleFactor();
+    const auto environ = amAppIface->environ();
+    const QStringList envs(environ.split(';'));
+    // return true if envs contains any one of DisabledScaleEnvironments.
+    auto iter = std::find_if(envs.begin(), envs.end(), [] (const QString &env) {
+        return DisabledScaleEnvironments.contains(env);
+    });
+    return iter != envs.end();
 }
 
-void AppMgr::setScaleFactor(const QString &desktopId, double scaleFactor)
+void AppMgr::setDisableScale(const QString &desktopId, bool disableScale)
 {
     AppManager1Application * amAppIface = createAM1AppIface(desktopId);
-    if (!amAppIface) return;
+    if (!amAppIface) {
+        qCWarning(logDdeIntegration) << "Failed to get interface for desktop ID:" << desktopId;
+        return;
+    }
 
-    amAppIface->setScaleFactor(scaleFactor);
+    QString environ = amAppIface->environ();
+    QStringList envs(environ.split(';', Qt::SkipEmptyParts));
+
+    if (disableScale) {
+        // remove all ScaleEnvironments, avoid other caller has set it manually.
+        envs.removeIf([] (const QString &env) {
+            auto iter = std::find_if(DisabledScaleEnvironments.begin(), DisabledScaleEnvironments.end(),
+                                     [env] (const QString &item) {
+                                         const auto left = item.split('=');
+                                         const auto right = env.split('=');
+                                         return !right.isEmpty() && left.at(0) == right.at(0);
+                                     });
+            return iter != DisabledScaleEnvironments.end();
+        });
+        envs << DisabledScaleEnvironments;
+    } else {
+        // remove all DisabledScaleEnvironments.
+        envs.removeIf([] (const QString &env) {
+            return DisabledScaleEnvironments.contains(env);
+        });
+    }
+
+    environ = envs.join(';');
+    qCDebug(logDdeIntegration) << "Update environ for the desktopId" << desktopId << ", env:" << environ;
+    amAppIface->setEnviron(environ);
 }
 
 bool AppMgr::isOnDesktop(const QString &desktopId)
 {
     AppManager1Application * amAppIface = createAM1AppIface(desktopId);
-    if (!amAppIface) return false;
+    if (!amAppIface) {
+        qCWarning(logDdeIntegration) << "Failed to get interface for desktop ID:" << desktopId;
+        return false;
+    }
 
     return amAppIface->isOnDesktop();
 }
@@ -210,29 +284,37 @@ bool AppMgr::isOnDesktop(const QString &desktopId)
 bool AppMgr::sendToDesktop(const QString &desktopId)
 {
     AppManager1Application * amAppIface = createAM1AppIface(desktopId);
-    if (!amAppIface) return false;
+    if (!amAppIface) {
+        qCWarning(logDdeIntegration) << "Failed to get interface for desktop ID:" << desktopId;
+        return false;
+    }
 
     QDBusPendingReply<bool> reply = amAppIface->SendToDesktop();
     reply.waitForFinished();
 
     if (reply.isError()) {
-        qDebug() << reply.error();
+        qCWarning(logDdeIntegration) << "SendToDesktop failed:" << reply.error();
         return false;
     }
 
-    return reply.value();
+    const bool result = reply.value();
+    qCInfo(logDdeIntegration) << "SendToDesktop result for" << desktopId << ":" << result;
+    return result;
 }
 
 bool AppMgr::removeFromDesktop(const QString &desktopId)
 {
     AppManager1Application * amAppIface = createAM1AppIface(desktopId);
-    if (!amAppIface) return false;
+    if (!amAppIface) {
+        qCWarning(logDdeIntegration) << "Failed to get interface for desktop ID:" << desktopId;
+        return false;
+    }
 
     QDBusPendingReply<bool> reply = amAppIface->RemoveFromDesktop();
     reply.waitForFinished();
 
     if (reply.isError()) {
-        qDebug() << reply.error();
+        qCWarning(logDdeIntegration) << "RemoveFromDesktop failed:" << reply.error();
         return false;
     }
 
@@ -267,41 +349,46 @@ void AppMgr::watchingAppItemPropertyChanged(const QString &key, AppMgr::AppItem
     Q_ASSERT(appItem->handler == nullptr);
     appItem->handler = amAppIface;
     connect(amAppIface, &AppManager1Application::CategoriesChanged, this, [this, appItem](const QStringList & value) {
-        qDebug() << "CategoriesChanged by AM, desktopId" << appItem->id;
+        qCDebug(logDdeIntegration) << "CategoriesChanged by AM, desktopId" << appItem->id;
         appItem->categories = value;
         Q_EMIT itemDataChanged(appItem->id);
     });
     connect(amAppIface, &AppManager1Application::IconsChanged, this, [this, appItem](const QStringMap & value) {
-        qDebug() << "IconsChanged by AM, desktopId" << appItem->id;
+        qCDebug(logDdeIntegration) << "IconsChanged by AM, desktopId" << appItem->id;
         appItem->iconName = parseIcon(value);
         Q_EMIT itemDataChanged(appItem->id);
     });
     connect(amAppIface, &AppManager1Application::X_Deepin_VendorChanged, this, [this, appItem, amAppIface](const QString & value) {
-        qDebug() << "X_Deepin_VendorChanged by AM, desktopId" << appItem->id;
+        qCDebug(logDdeIntegration) << "X_Deepin_VendorChanged by AM, desktopId" << appItem->id;
         appItem->displayName = getDisplayName(!value.isEmpty(), amAppIface->name(), amAppIface->genericName());
         Q_EMIT itemDataChanged(appItem->id);
     });
     connect(amAppIface, &AppManager1Application::GenericNameChanged, this, [this, appItem, amAppIface](const QStringMap & value) {
-        qDebug() << "GenericNameChanged by AM, desktopId" << appItem->id;
+        qCDebug(logDdeIntegration) << "GenericNameChanged by AM, desktopId" << appItem->id;
         appItem->displayName = getDisplayName(!amAppIface->x_Deepin_Vendor().isEmpty(), amAppIface->name(), value);
         Q_EMIT itemDataChanged(appItem->id);
     });
     connect(amAppIface, &AppManager1Application::NameChanged, this, [this, appItem, amAppIface](const QStringMap & value) {
-        qDebug() << "NameChanged by AM, desktopId" << appItem->id;
+        qCDebug(logDdeIntegration) << "NameChanged by AM, desktopId" << appItem->id;
         appItem->name = parseName(value);
         appItem->displayName = getDisplayName(!amAppIface->x_Deepin_Vendor().isEmpty(), value, amAppIface->genericName());
         Q_EMIT itemDataChanged(appItem->id);
     });
     connect(amAppIface, &AppManager1Application::InstalledTimeChanged, this, [this, appItem](const qint64 & value) {
-        qDebug() << "InstalledTimeChanged by AM, desktopId" << appItem->id;
+        qCDebug(logDdeIntegration) << "InstalledTimeChanged by AM, desktopId" << appItem->id;
         appItem->installedTime = value;
         Q_EMIT itemDataChanged(appItem->id);
     });
     connect(amAppIface, &AppManager1Application::LastLaunchedTimeChanged, this, [this, appItem](const qint64 & value) {
-        qDebug() << "LastLaunchedTimeChanged by AM, desktopId" << appItem->id;
+        qCDebug(logDdeIntegration) << "LastLaunchedTimeChanged by AM, desktopId" << appItem->id;
         appItem->lastLaunchedTime = value;
         Q_EMIT itemDataChanged(appItem->id);
     });
+    connect(amAppIface, &AppManager1Application::AutoStartChanged, this, [this, appItem](bool value) {
+        qCDebug(logDdeIntegration) << "AutoStartChanged by AM, desktopId" << appItem->id;
+        appItem->isAutoStart = value;
+        Q_EMIT itemDataChanged(appItem->id);
+    });
 }
 
 void AppMgr::updateAppsLaunchedTimes(const QVariantMap &appsLaunchedTimes)
@@ -316,7 +403,7 @@ void AppMgr::updateAppsLaunchedTimes(const QVariantMap &appsLaunchedTimes)
 
         // including reset and increase times.
         if (item->launchedTimes != times) {
-            qDebug() << "LaunchedTimesChanged by DConfig, desktopId" << item->id;
+            qCDebug(logDdeIntegration) << "LaunchedTimesChanged by DConfig, desktopId" << item->id << "from" << item->launchedTimes << "to" << times;
             item->launchedTimes = times;
             Q_EMIT itemDataChanged(item->id);
         }
@@ -325,26 +412,29 @@ void AppMgr::updateAppsLaunchedTimes(const QVariantMap &appsLaunchedTimes)
 
 void AppMgr::initObjectManager()
 {
-    if (!isValid())
+    if (!isValid()) {
+        qCWarning(logDdeIntegration) << "Object manager is not valid, aborting initialization";
         return;
+    }
 
     connect(m_objectManager, &AppManager1ApplicationObjectManager::InterfacesAdded, this,
             [this](const QDBusObjectPath &objPath, ObjectInterfaceMap interfacesAndProperties) {
                 const QString key(objPath.path());
-                qDebug() << "InterfacesAdded by AM, path:" << key;
+                qCDebug(logDdeIntegration) << "InterfacesAdded by AM, path:" << key;
                 if (m_appItems.contains(objPath.path())) {
                     qWarning() << "App already exists for the path:" << key;
                     return;
                 }
                 if (auto appItem = parseDBus2AppItem(interfacesAndProperties)) {
-                    qDebug() << "App item added, desktopId" << appItem->id;
+                    qCDebug(logDdeIntegration) << "App item added, desktopId" << appItem->id;
                     watchingAppItemAdded(key, appItem);
                 }
             });
     connect(m_objectManager, &AppManager1ApplicationObjectManager::InterfacesRemoved, this,
             [this](const QDBusObjectPath &objPath, const QStringList &interfaces) {
+                Q_UNUSED(interfaces)
                 const QString key(objPath.path());
-                qDebug() << "InterfacesRemoved by AM, path:" << key;
+                qCDebug(logDdeIntegration) << "InterfacesRemoved by AM, path:" << key;
                 watchingAppItemRemoved(key);
             });
 
@@ -352,16 +442,18 @@ void AppMgr::initObjectManager()
 
     DConfig *config = DConfig::create("org.deepin.dde.application-manager", "org.deepin.dde.am", "", this);
     if (!config->isValid()) {
-        qWarning() << "DConfig is invalid when getting launched times.";
+        qCWarning(logDdeIntegration) << "DConfig is invalid when getting launched times.";
     } else {
         static const QString AppsLaunchedTimes(u8"appsLaunchedTimes");
         const auto &value = config->value(AppsLaunchedTimes).toMap();
         updateAppsLaunchedTimes(value);
         QObject::connect(config, &DConfig::valueChanged, this, [this, config](const QString &key) {
-            if (key != AppsLaunchedTimes)
+            if (key != AppsLaunchedTimes) {
+                qCDebug(logDdeIntegration) << "Ignoring non-appsLaunchedTimes key:" << key;
                 return;
+            }
 
-            qDebug() << "appsLaunchedTimes of DConfig Changed.";
+            qCInfo(logDdeIntegration) << "appsLaunchedTimes of DConfig changed, updating";
             const auto &value = config->value(AppsLaunchedTimes).toMap();
             updateAppsLaunchedTimes(value);
         });
@@ -370,7 +462,7 @@ void AppMgr::initObjectManager()
 
 void AppMgr::fetchAppItems()
 {
-    qDebug() << "Begin to fetch apps.";
+    qCDebug(logDdeIntegration) << "Begin to fetch apps.";
     const auto reply = m_objectManager->GetManagedObjects();
     QDBusPendingCallWatcher *watcher = new QDBusPendingCallWatcher(reply, this);
     connect(watcher, &QDBusPendingCallWatcher::finished, this, [this](QDBusPendingCallWatcher *call){
@@ -380,7 +472,7 @@ void AppMgr::fetchAppItems()
             call->deleteLater();
             return;
         }
-        qDebug() << "Fetched all AppItem, and start parsing data.";
+        qCDebug(logDdeIntegration) << "Fetched all AppItem, and start parsing data.";
         QMap<QString, AppMgr::AppItem *> items;
         const auto objects = reply.value();
         for (auto iter = objects.cbegin(); iter != objects.cend(); ++iter) {
@@ -395,7 +487,7 @@ void AppMgr::fetchAppItems()
             watchingAppItemPropertyChanged(objPath.path(), appItem);
         }
         call->deleteLater();
-        qDebug() << "Fetched all AppItem, and end up parsing data.";
+        qCDebug(logDdeIntegration) << "Fetched all AppItem, and end up parsing data.";
 
         m_appItems = items;
         Q_EMIT changed();
@@ -414,11 +506,14 @@ void AppMgr::watchingAppItemAdded(const QString &key, AppItem *appItem)
 void AppMgr::watchingAppItemRemoved(const QString &key)
 {
     auto appItem = m_appItems.value(key);
-    if (!appItem)
+    if (!appItem) {
+        qCWarning(logDdeIntegration) << "App item not found for key:" << key;
         return;
+    }
 
-    qDebug() << "App item removed, desktopId" << appItem->id;
+    qCDebug(logDdeIntegration) << "App item removed, desktopId" << appItem->id;
     if (auto handler = appItem->handler) {
+        qCDebug(logDdeIntegration) << "Deleting handler for removed app:" << appItem->id;
         handler->deleteLater();
     }
     m_appItems.remove(key);
